/**

OtherPairs

From a given currency pair, determine the base currency, then determine how many of
the pairs have 20EMA above 50EMA.

In cases that the base currency is actually the quote currency of the test pair,
reverse the order, i.e. 20EMA below 50EMA.

For example: USDCAD has base currency USD.
* If USDCAD has 20EMA above 50EMA, that's +1.
* If EURUSD has 20EMA below 50EMA, that's also +1, because USD is the quote currency.

*/

#property copyright		"2018 Anthony J. Garot"
#property link			"http://www.garot.com/trading/"
#property version		"1.01"
#property description	"This indicator determines . . . "
#property description	" "
#property description	"MQL5 Profile: https://www.mql5.com/en/users/tonegarot"
#property description	"My Website: http://www.garot.com/trading/"
#property strict

//--- indicator settings
#property indicator_separate_window
//#property indicator_chart_window

#property indicator_minimum		-7.02				// I have this 1.02 + the maximum it can be
#property indicator_maximum		+7.02

#property indicator_height		175

#property indicator_buffers 	2
#property indicator_plots		1

#property indicator_type1		DRAW_COLOR_HISTOGRAM
#property indicator_color1		CLR_NONE,SeaGreen,FireBrick,Yellow
#property indicator_width1		2

struct SPair
{
	string	symbol;
	int		direction;			// +1 or -1
	int		calculatedFast;
	int		calculatedSlow;
	// handles
	int		FastHandle;
	int		MediumHandle;
	// MA's
	double	FastBuffer[];		// internal buffer
	double	MediumBuffer[];		// internal buffer

	SPair()
	{
		calculatedFast = 0;
		calculatedSlow = 0;
	}
};

// Note: this matches the order of the majors[] array above.
enum EnMajors
{
	MAJOR_EURUSD,	// EURUSD
	MAJOR_USDJPY,	// USDJPY
	MAJOR_GBPUSD,	// GBPUSD
	MAJOR_USDCAD,	// USDCAD
	MAJOR_USDCHF,	// USDCHF
	MAJOR_AUDUSD,	// AUDUSD
	MAJOR_NZDUSD	// NZDUSD
};

//---- constants
const int 					MAIN_WINDOW = 0;
const int 					SUB_WINDOW = 1;
const int 					chartID = 0;
const int					START_POS = 0;
const double				COLOR_NONE = 0.;
const double				COLOR_POSITIVE = 1.;
const double				COLOR_NEGATIVE = 2.;
const double				COLOR_CLEAR_MAJORITY = 3.;
const ENUM_APPLIED_PRICE	APPLIED_PRICE=PRICE_CLOSE;	// Applied Price
const int					NUM_SYMBOLS = 7;
const string majors[] = {
	"EURUSD",					// 7 of 7 share EUR as base currency
	"USDJPY",					// 3 of 7 share USD as base currency
	"GBPUSD",					// 6 of 7 share GBP as base currency
	"USDCAD",
	"USDCHF",
	"AUDUSD",					// 5 of 7 share AUD as base currency
	"NZDUSD"					// 4 of 7 share NZD as base currency
};
// This is all the possible combinations of the 6 major currencies.
// There are 28 of them. It doesn't consider PLN, SGD, NOK, SEK, DKK, etc.,
// even though those are in the minors.
//
// EUR / USD
//		EUR == base currency
//		USD == quote currency
const string currencyPairs[] = {
	"EURUSD",		// major
	"GBPUSD",		// major
	"USDJPY",		// major
	"EURGBP",		// major
	"EURJPY",		// major
	"GBPJPY",		// major
	"USDCAD",		// major
	"CADCHF",		// major
	"USDCHF",		// major
	"CHFJPY",		// major
	"EURCHF",		// major
	"GBPCHF",		// major
	"CADJPY",		// major
	"EURCAD",		// major
	"GBPCAD",		// major
	"AUDUSD",		//	minor
	"NZDUSD",		//	minor
	"AUDCHF",		//	minor
	"NZDCHF",		//	minor
	"AUDCAD",		//	minor
	"AUDJPY",		//	minor
	"AUDNZD",		//	minor
	"EURAUD",		//	minor
	"EURNZD",		//	minor
	"GBPAUD",		//	minor
	"GBPNZD",		//	minor
	"NZDCAD",		//	minor
	"NZDJPY"		//	minor
};

//---- input parameters
//input string			InpCurrencyPair="USDCAD" /* base = USD; quote = CAD */;
input EnMajors			InpCurrencyPair = MAJOR_USDCAD;	// Currency Pair
input int				InpMaFastPeriod=20;				// Fast MA period
input int				InpMaMediumPeriod=50;			// Medium MA period
input ENUM_MA_METHOD	InpMaFastMethod=MODE_EMA;		// Fast MA Method
input ENUM_MA_METHOD	InpMaMediumMethod=MODE_EMA;		// Medium MA Method

//---- indicator buffers
// external buffers
double	ExtHistoBuffer[];
double	ExtHistoColors[];


//---- globals
ENUM_TIMEFRAMES	g_chartTimeFrame = PERIOD_D1;
int				g_barsMinimum = -1;			// Minimum number of bars to perform a calculation?
SPair			pairs[7];					// Other pairs (NUM_SYMBOLS)
string			g_baseCurrency;
int				g_debugBar = 0;				// set to an index to turn DEBUG on for that bar
int				DEBUG = 0 /* The greater the number, the more verbose. */;	// DEBUG 0=off;1=verbose;2=spray;3=diarrhea


/**
 * Per whroeder1
 * Don't try to use any price or server related functions in OnInit (or on load,)
 * as there may be no connection/chart yet
 */
int OnInit()
{
	if ( DEBUG > 1 ) PrintFormat("%s(): symbol [%s] max bars [%d]", __FUNCTION__, Symbol(), TerminalInfoInteger(TERMINAL_MAXBARS) );
	//return INIT_FAILED;

	g_baseCurrency = StringSubstr(majors[InpCurrencyPair],0,3);
	if ( DEBUG > 1 ) PrintFormat("g_baseCurrency [%s]", g_baseCurrency);
	Comment(StringFormat("Base currency [%s]", g_baseCurrency));

	string shortName = StringFormat("TommorOtherPairs (%d,%d)", InpMaFastPeriod, InpMaMediumPeriod);
	if ( DEBUG ) PrintFormat("%s(): %s", __FUNCTION__, shortName);

	// bars minimum for calculation
	g_barsMinimum = InpMaFastPeriod;
	g_barsMinimum = MathMax(g_barsMinimum, InpMaMediumPeriod);
	if ( DEBUG ) PrintFormat("g_barsMinimum [%d]", g_barsMinimum);

	// The problem with this is the calculation is all wrong.
	// This is the expedient solution, but it doesn't work in the long run.
	ArraySetAsSeries(ExtHistoBuffer, true);
	ArraySetAsSeries(ExtHistoColors, true);

	//---- indicator buffers mapping
	SetIndexBuffer(0,ExtHistoBuffer,INDICATOR_DATA);
	SetIndexBuffer(1,ExtHistoColors,INDICATOR_COLOR_INDEX);

	PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,g_barsMinimum);
	PlotIndexSetString(0,PLOT_LABEL,"HISTO");
	PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,0);			// Set as an empty value 0
	PlotIndexSetString(1,PLOT_LABEL,"COLOR");

	IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
	IndicatorSetString(INDICATOR_SHORTNAME,shortName);

	if ( ! pLoadRelevantPairs() )
	{
		return (INIT_FAILED);
	}

	// initialization done
	if ( DEBUG > 1 ) Print("initialization succeeded");
	return(INIT_SUCCEEDED);
}

// Use the long form so as not to get the Parameters tab
int OnCalculate (const int rates_total,      // size of input time series
                 const int prev_calculated,  // bars handled in previous call
                 const datetime& time[],     // Time
                 const double& open[],       // Open
                 const double& high[],       // High
                 const double& low[],        // Low
                 const double& close[],      // Close
                 const long& tick_volume[],  // Tick Volume
                 const long& volume[],       // Real Volume
                 const int& spread[]         // Spread
   )
{
	// check for rates total
	if(rates_total<g_barsMinimum)
	{
		if ( DEBUG ) PrintFormat("NOT ENOUGH: rates_total [%d] < minimum [%d]", rates_total, g_barsMinimum );
		return(0); // not enough bars for calculation
	}

	// we don't have to copy all of the data all the time
	int to_copy;
	if(prev_calculated>rates_total || prev_calculated<0)
	{
		to_copy=rates_total;
	}
	else
	{
		to_copy=rates_total-prev_calculated;
		if(prev_calculated>0) to_copy++;
	}
	if ( DEBUG > 1 ) PrintFormat("to_copy [%d] rates_total [%d] prev_calculated [%d]",to_copy,rates_total,prev_calculated);

	// Ensure we have enough data for all other currency pairs
	if ( ! IsWaiting(4000) )
	{
		if ( DEBUG ) Print("Not waiting!");
		if ( ! pEnsureData(to_copy, time[1]) )
		{
			if ( DEBUG ) PrintFormat("pEnsureData . . . returning to_copy [%d] which will become prev_calculated", to_copy);
			return 0;
		}
		else
		{
			if ( ! pFillBuffers(to_copy) )
			{
				if ( DEBUG ) PrintFormat("pFillBuffers . . . returning to_copy [%d] which will become prev_calculated", to_copy);
				return 0;
			}
		}
	}

	// correct position, when it's first iteration
	int pos = prev_calculated - 1;
	if ( pos <= 0 )
	{
		pos = 1;

		// Set up the 0-values
		ExtHistoBuffer[0] = 0.;
		ExtHistoColors[0] = COLOR_NONE;
	}

	//--- the main loop of calculations
	for(int i=pos;i<rates_total - 1 && !IsStopped();i++)
	{
		// Start with nuthin'
		ExtHistoBuffer[i] = 0.;
		ExtHistoColors[i] = COLOR_NONE;

		// skip values where slower MA's haven't fully loaded a cycle
		// Unnecessary when ArraySetAsSeries() flag is true
		// if ( i < g_barsMinimum ) { break; }

		if ( g_debugBar > 0 )
		{
			if ( i == g_debugBar )
			{
				DEBUG=4;
			}
			else
			{
				DEBUG=0;
			}
		}

		// 11. Does a clear majority of other pairs with same base currency
		// (first currency of pair) also show 20EMA above 50EMA?
		for ( int j=0; j<NUM_SYMBOLS;j++ )
		{
			if ( pairs[j].FastBuffer[i] > pairs[j].MediumBuffer[i] )
			{
				if ( pairs[j].direction == 1 )
				{
					ExtHistoBuffer[i]++;
					if ( DEBUG > 2 ) PrintFormat("Fast Above +1: i [%d] j [%d] symbol [%s] dir [%d] histo [%.0f]",
						i,
						j,
						pairs[j].symbol,
						pairs[j].direction,
						ExtHistoBuffer[i] );
				}
				else
				{
					ExtHistoBuffer[i]--;
					if ( DEBUG > 2 ) PrintFormat("Fast Above -1: i [%d] j [%d] symbol [%s] dir [%d] histo [%.0f]",
						i,
						j,
						pairs[j].symbol,
						pairs[j].direction,
						ExtHistoBuffer[i] );
				}
			}
			else if ( pairs[j].FastBuffer[i] < pairs[j].MediumBuffer[i]  )
			{
				if ( pairs[j].direction == -1 )
				{
					ExtHistoBuffer[i]++;
					if ( DEBUG > 2 ) PrintFormat("Fast Below +1: i [%d] j [%d] symbol [%s] dir [%d] histo [%.0f]",
						i,
						j,
						pairs[j].symbol,
						pairs[j].direction,
						ExtHistoBuffer[i] );
				}
				else
				{
					ExtHistoBuffer[i]--;
					if ( DEBUG > 2 ) PrintFormat("Fast Below -1: i [%d] j [%d] symbol [%s] dir [%d] histo [%.0f]",
						i,
						j,
						pairs[j].symbol,
						pairs[j].direction,
						ExtHistoBuffer[i] );
				}
			}
			else
			{
				PrintFormat("what? %f %f", pairs[j].FastBuffer[i], pairs[j].MediumBuffer[i]);
			}
		}

		if ( ExtHistoBuffer[i] > 4. )
		{
			ExtHistoColors[i] = COLOR_CLEAR_MAJORITY;
		}
		else if ( ExtHistoBuffer[i] > 0. )
		{

			ExtHistoColors[i] = COLOR_POSITIVE;
		}
		else if ( ExtHistoBuffer[i] < 0. )
		{
			ExtHistoColors[i] = COLOR_NEGATIVE;
		}
		else
		{
			ExtHistoColors[i] = COLOR_NONE;
		}
	}

	// return value of prev_calculated for next call
	if ( DEBUG > 1 ) PrintFormat("Returning rates_total [%d]", rates_total);
	return(rates_total);
}

/**
 * Remember: Do this in OnCalculate(), not in OnInit(). See note there for more info.
 */
bool pEnsureData(const int to_copy, const datetime currentTime)
{
	if ( DEBUG > 1 ) PrintFormat("%s(): Entered. to_copy [%d] currentTime [%s]",
								__FUNCTION__,
								to_copy,
								TimeToString(currentTime) );

	bool isSynchronized = false;
	while ( ! isSynchronized )
	{
		isSynchronized = true;		// until proven guilty

		// Loop over all symbols
		for ( int j=0; j<NUM_SYMBOLS;j++ )
		{
			if(IsStopped()) return false; //Checking for stop flag

			// not all data may be calculated
			int calculated;
			ResetLastError();
			calculated=BarsCalculated(pairs[j].FastHandle);

			// sometimes happens for some reason
			if ( calculated == 0 && pairs[j].calculatedFast > 0 )
			{
				PrintFormat("[%s] 0 calculated for some reason", pairs[j].symbol);
				isSynchronized = false;
				break;
			}

			if(calculated<to_copy)
			{
				// Need more calculated
				PrintFormat("BarsCalculated() loading data for MA (%d) [%s] (%d bars < %d needed). Code [%d].",
							InpMaFastPeriod,
							pairs[j].symbol,
							calculated,
							to_copy,
							GetLastError());

				if ( pairs[j].symbol != Symbol() )
				{
					WrapCopyTimeSingle(currentTime, pairs[j].symbol, 0);
					isSynchronized = false;
					break;
				}
			}
			else
			{
				pairs[j].calculatedFast = calculated;

				if ( DEBUG > 1 ) PrintFormat(
					"BarsCalculated() OK. (%d of %d needed) for symbol [%s]",
					calculated,
					to_copy,
					pairs[j].symbol );
			}

			ResetLastError();
			calculated=BarsCalculated(pairs[j].MediumHandle);

			// sometimes happens for some reason
			if ( calculated == 0 && pairs[j].calculatedSlow > 0 )
			{
				PrintFormat("[%s] 0 calculated for some reason", pairs[j].symbol);
				isSynchronized = false;
				break;
			}

			if(calculated<to_copy)
			{
				// Need more calculated
				PrintFormat("BarsCalculated() loading data for MA (%d) [%s] (%d bars < %d needed). Code [%d].",
							InpMaMediumPeriod,
							pairs[j].symbol,
							calculated,
							to_copy,
							GetLastError());

				if ( pairs[j].symbol != Symbol() )
				{
					WrapCopyTimeSingle(currentTime, pairs[j].symbol, 0);
					isSynchronized = false;
					break;
				}
			}
			else
			{
				pairs[j].calculatedSlow = calculated;

				if ( DEBUG > 1 ) PrintFormat(
					"BarsCalculated() OK. (%d of %d needed) for symbol [%s]",
					calculated,
					to_copy,
					pairs[j].symbol );
			}
		}
	}

	if ( DEBUG > 1 ) Print("Synchronized!");
	return true;
}

bool pFillBuffers(const int to_copy)
{
	if ( DEBUG > 1 && to_copy > 1 ) PrintFormat("%s(): START_POS [%d] to_copy [%d]", __FUNCTION__, START_POS, to_copy);

	bool gotIssue = false;
	for ( int j=0; j<NUM_SYMBOLS;j++ )
	{
		int copied;
		copied = CopyBuffer(pairs[j].FastHandle,0,START_POS,to_copy,pairs[j].FastBuffer);
		if( copied < to_copy )
		{
			PrintFormat("CopyBuffer() loading for iMA(%d) [%s] (%d<%d) Code [%d].", InpMaFastPeriod, pairs[j].symbol, copied, to_copy, GetLastError());
			gotIssue = true;
		}
		if ( DEBUG > 1 && copied > 1 ) PrintFormat("CopyBuffer() OK for iMA(%d) [%s] returned [%d].", InpMaFastPeriod, pairs[j].symbol, copied);

		copied = CopyBuffer(pairs[j].MediumHandle,0,START_POS,to_copy,pairs[j].MediumBuffer);
		if( copied < to_copy )
		{
			PrintFormat("CopyBuffer() loading for iMA(%d) [%s] (%d<%d) Code [%d].", InpMaMediumPeriod, pairs[j].symbol, copied, to_copy, GetLastError());
			gotIssue = true;
		}
		if ( DEBUG > 1 && copied > 1 ) PrintFormat("CopyBuffer() OK for iMA(%d) [%s] returned [%d].", InpMaMediumPeriod, pairs[j].symbol, copied);
	}
	if ( gotIssue ) return false;

	if ( DEBUG > 1 ) Print("Buffers Filled!");
	return true;
}

// Get the 7 symbols for the base currency.
bool pLoadRelevantPairs(void)
{
	int j = 0;
	for(int i = 0; i<ArraySize(currencyPairs);i++)
	{
		bool gotOne = false;

		string base = StringSubstr(currencyPairs[i],0,3);
		string quote = StringSubstr(currencyPairs[i],3,3);
		if ( base == g_baseCurrency )
		{
			pairs[j].symbol = currencyPairs[i];
			pairs[j].direction = 1;
			if ( DEBUG > 1 ) PrintFormat("[%d] Symbol [%s] is included [%d]", j, pairs[j].symbol, pairs[j].direction);
			gotOne = true;
		} else if ( quote == g_baseCurrency )
		{
			pairs[j].symbol = currencyPairs[i];
			pairs[j].direction = -1;
			if ( DEBUG > 1 ) PrintFormat("[%d] Symbol [%s] is included [%d]", j, pairs[j].symbol, pairs[j].direction);
			gotOne = true;
		}

		if ( gotOne )
		{
			// get handles for the indicators used in this indicator
			pairs[j].FastHandle=iMA(
				pairs[j].symbol,	// symbol
				0,					// period
				InpMaFastPeriod,	// ma_period
				0,					// ma_shift
				InpMaFastMethod,	// ma_method
				APPLIED_PRICE		// applied_price
				);
			if ( pairs[j].FastHandle == INVALID_HANDLE ) { Print(StringFormat("Error creating indicator handle for InpMaFastPeriod [%d]", InpMaFastPeriod)); return false; }

			pairs[j].MediumHandle=iMA(
				pairs[j].symbol,	// symbol
				0,					// period
				InpMaMediumPeriod,	// ma_period
				0,					// ma_shift
				InpMaMediumMethod,	// ma_method
				APPLIED_PRICE		// applied_price
				);
			if ( pairs[j].MediumHandle == INVALID_HANDLE ) { Print(StringFormat("Error creating indicator handle for InpMaMediumPeriod [%d]", InpMaMediumPeriod)); return false; }

			j++;
		}
	}

	return true;
}

// GetTickCount() is the number of milliseconds that elapsed since the system start.
// So add our millseconds to it.
bool IsWaiting(const int milliseconds)
{
	static uint untilTime = 0;				// First time we aren't waiting

	if ( GetTickCount() < untilTime )
	{
		return true;
	}

	// No longer waiting on this cycle, so reset
	untilTime = GetTickCount() + milliseconds;
	return false;
}

/**
Attempt to copy the time.
https://www.mql5.com/en/docs/series/copytime
Into an array created for this purpose array (tmpTimeArray), copy the opening
of the zero bar of each affected instrument. CopyTime will cause a pull of more data.
ERR_HISTORY_NOT_FOUND	4401	Requested history not found

For Weekly, CopyTime() will always return the Sunday value.
*/
datetime WrapCopyTimeSingle(const datetime bartime, const string symbol = NULL, const ENUM_TIMEFRAMES tf = 0)
{
	datetime tmpTimeArray[1];	// Intermediate array for the time of the bar.

	ResetLastError();
	int copied = CopyTime(symbol,tf,bartime,1,tmpTimeArray);	// Requesting only 1 bar!

	if ( copied == -1 )
	{
		if ( DEBUG > 2 )
		{
			// This is NOT an error. This is expected behavior while data
			// is pulled from the broker. Even so, I made it DEBUG level 1.
			PrintFormat("%s(): count [-1] symbol [%s] TF [%s] bartime [%s]. Error [%d]",
						__FUNCTION__,
						symbol,
						EnumToString(tf),
						TimeToString(bartime),
						GetLastError());
		}

		// This is OK. I'll just let OnCalculate() know, and it will run that bar again.
		return NULL;
	}

	if ( DEBUG > 2 )
	{
		PrintFormat("%s(): [%s] copied count [%d] TF [%s] value [%s]",
					__FUNCTION__,
					TimeToString(bartime),
					copied,
					EnumToString(tf),
					TimeToString(tmpTimeArray[0]));
	}

	return tmpTimeArray[0];
}
